package com.sun.corba.se.impl.presentation.rmi;

import org.omg.CORBA.portable.ServantObject;
import com.sun.corba.se.spi.presentation.rmi.DynamicMethodMarshaller;
import java.lang.reflect.InvocationTargetException;
import java.security.AccessController;
import java.security.PrivilegedAction;
import com.sun.corba.se.spi.orb.ORB;
import org.omg.CORBA.portable.RemarshalException;
import org.omg.CORBA.portable.ApplicationException;
import org.omg.CORBA_2_3.portable.InputStream;
import org.omg.CORBA_2_3.portable.OutputStream;
import org.omg.CORBA.SystemException;
import javax.rmi.CORBA.Util;
import java.lang.reflect.Method;
import com.sun.corba.se.pept.transport.ContactInfoList;
import org.omg.CORBA.portable.Delegate;
import com.sun.corba.se.spi.transport.CorbaContactInfoList;
import com.sun.corba.se.spi.protocol.CorbaClientDelegate;
import com.sun.corba.se.spi.presentation.rmi.StubAdapter;
import java.lang.reflect.Proxy;
import org.omg.CORBA.Object;
import com.sun.corba.se.spi.presentation.rmi.PresentationManager;
import com.sun.corba.se.spi.orbutil.proxy.LinkedInvocationHandler;

public final class StubInvocationHandlerImpl implements LinkedInvocationHandler
{
    private transient PresentationManager.ClassData classData;
    private transient PresentationManager pm;
    private transient org.omg.CORBA.Object stub;
    private transient Proxy self;
    
    public void setProxy(Proxy self) {
        this.self = self;
    }
    
    public Proxy getProxy() {
        return this.self;
    }
    
    public StubInvocationHandlerImpl(PresentationManager pm, PresentationManager.ClassData classData, org.omg.CORBA.Object stub) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new DynamicAccessPermission("access"));
        }
        this.classData = classData;
        this.pm = pm;
        this.stub = stub;
    }
    
    private boolean isLocal() {
        boolean useLocalInvocation = false;
        Delegate delegate = StubAdapter.getDelegate(this.stub);
        if ((delegate instanceof CorbaClientDelegate)) {
            ContactInfoList contactInfoList = ((CorbaClientDelegate)delegate).getContactInfoList();
            if ((contactInfoList instanceof CorbaContactInfoList)) {
                useLocalInvocation = ((CorbaContactInfoList)contactInfoList).getLocalClientRequestDispatcher().useLocalInvocation(null);
            }
        }
        return useLocalInvocation;
    }
    
    public Object invoke(Object o, final Method method, Object[] array) throws Throwable {
        String idlName = this.classData.getIDLNameTranslator().getIDLName(method);
        DynamicMethodMarshaller dynamicMethodMarshaller = this.pm.getDynamicMethodMarshaller(method);
        Delegate delegate;
        try {
            delegate = StubAdapter.getDelegate(this.stub);
        }
        catch (SystemException ex) {
            throw Util.mapSystemException(ex);
        }
        if (!this.isLocal()) {
            try {
                InputStream inputStream = null;
                try {
                    OutputStream outputStream = (OutputStream)delegate.request(this.stub, idlName, true);
                    dynamicMethodMarshaller.writeArguments(outputStream, array);
                    inputStream = (InputStream)delegate.invoke(this.stub, outputStream);
                    return dynamicMethodMarshaller.readResult(inputStream);
                }
                catch (ApplicationException ex2) {
                    throw dynamicMethodMarshaller.readException(ex2);
                }
                catch (RemarshalException ex3) {
                    return this.invoke(o, method, array);
                }
                finally {
                    delegate.releaseReply(this.stub, inputStream);
                }
            }
            catch (SystemException ex4) {
                throw Util.mapSystemException(ex4);
            }
        }
        ORB orb = (ORB)delegate.orb(this.stub);
        ServantObject servant_preinvoke = delegate.servant_preinvoke(this.stub, idlName, method.getDeclaringClass());
        if (servant_preinvoke == null) {
            return this.invoke(this.stub, method, array);
        }
        try {
            Object[] copyArguments = dynamicMethodMarshaller.copyArguments(array, orb);
            if (!method.isAccessible()) {
                AccessController.doPrivileged(new PrivilegedAction() {
                    @Override
                    public Object run() {
                        method.setAccessible(true);
                        return null;
                    }
                });
            }
            return dynamicMethodMarshaller.copyResult(method.invoke(servant_preinvoke.servant, copyArguments), orb);
        }
        catch (InvocationTargetException ex5) {
            Throwable t = (Throwable)Util.copyObject(ex5.getCause(), orb);
            if (dynamicMethodMarshaller.isDeclaredException(t)) {
                throw t;
            }
            throw Util.wrapException(t);
        }
        catch (Throwable t2) {
            if ((t2 instanceof ThreadDeath)) {
                throw (ThreadDeath)t2;
            }
            throw Util.wrapException(t2);
        }
        finally {
            delegate.servant_postinvoke(this.stub, servant_preinvoke);
        }
    }
}
